# Шаг 3. Пользователи, и все, что с ними связано

* Пользователи могут регистрироваться в нашем магазине.
* Панель управления должна содержать средства для просмотра списка пользователей, и редактирования их.
* Пользователи будут проходить процедуру аутентификации, после этого идентификатор зарегистрированного пользователя будет храниться в сессии. Данные пользователя будут при необходимости автоматически подгружаться из базы данных.
* Профайл текущего пользователя будет отображаться в левой части страницы.

Все это требует новых знаний по устройству Limb, поэтому мы решили разбить эту страницу на несколько дополнительных шагов, где постарались все подробно объяснить.

## Класс User
Для начала нам необходима простейщая реализация модели пользователя - класс User, соответствующая требованиям:

* свойства login, name и email являются обязательными
* свойство email является правильным email-адресом

Файл shop/src/model/User.class.php:

    class User extends lmbActiveRecord 
    {
      protected $password;
 
      protected function _createValidator()
      {
        $validator = new lmbValidator();
 
        $validator->addRequiredRule('login');
        $validator->addRequiredRule('email');
        $validator->addRequiredRule('name');
 
        lmb_require('limb/validation/src/rule/lmbEmailRule.class.php');
        $validator->addRule(new lmbEmailRule('email'));
        return $validator;
      }
 
      protected function _onBeforeSave()
      {
        $this->_generatePassword();
      }
 
      protected function _generatePassword()
      {
        if($this->password)
          $this->setHashedPassword(self :: cryptPassword($this->password));
      }
 
      static function cryptPassword($password)
      {
        return md5($password);
      }
 
    }

Т.к. имя класса совпадает с именем таблицы в БД, то имени таблицы указывать не надо.

Мы перекрыли в классе фабричный метод **_createValidator()**. Этот метод возвращает валидатор, который используется для проверки данных внутри объектов Product при сохранении. Подробнее о валидации данных в классах ACTIVE_RECORD в разделе [«Валидация данных в объектах»](../../../../active_record/docs/ru/active_record/validation.md). Создаваемый в методе _createValidator() валидатор проверяет, что поля email, login и name должны быть заполнены, а поле email является валидным электронным адрессом.

Обратите внимание на метод **_onBeforeSave()**. Это так называемый метод расширения класса lmbActiveRecord, который дочерние классы могут использовать чтобы выполнять необходимые действия, например, перед сохранением новой записи, перед обновлением существующей, после сохранения и т.д. Список методов расширения класса lmbActiveRecord есть в разделе [«Расширение поведения базового класса lmbActiveRecord»](../../../../active_record/docs/ru/active_record/enhancement.md).

В нашем случае мы использовали этот метод, чтобы шифровать поле **password** и формировать значение поля **hashed_password**. Таким образом мы будем хранить только хешированный пароль, а обновление хешированного пароля будет происходить только, если в поле password есть новое значение.

## Далее

* [Шаг 3.1 Регистрация пользователей](./step3-1.md)
* [Шаг 3.2 Аутентификация пользователей](./step3-2.md)
* [Шаг 3.3 Редактирование и отображение профайла пользователя](./step3-3.md)
* [Шаг 3.4 Управление списком пользователей](./step3-4.md)
